@@ -22,29 +22,39 @@
 
 #include "backend.h"
 
-#include "audiooutput.h"
-#include "audiodataoutput.h"
-#include "mediaobject.h"
-#include "videowidget.h"
-#include "devicemanager.h"
-#include "effectmanager.h"
-#include "effect.h"
-#include "sinknode.h"
-//#include "videodataoutput.h"
-#include "vlcloader.h"
-#include "vlcmediaobject.h"
-
-#ifndef PHONON_VLC_NO_EXPERIMENTAL
-#include "experimental/avcapture.h"
-#endif // PHONON_VLC_NO_EXPERIMENTAL
+#include <QtCore/QCoreApplication>
+#include <QtCore/QDir>
+#include <QtCore/QFileInfo>
+#include <QtCore/QLatin1Literal>
+#include <QtCore/QLibrary>
+#include <QtCore/QSet>
+#include <QtCore/QSettings>
+#include <QtCore/QStringBuilder>
+#include <QtCore/QtPlugin>
+#include <QtCore/QVariant>
+#include <QtGui/QMessageBox>
 
 #ifdef PHONON_PULSESUPPORT
 #include <phonon/pulsesupport.h>
 #endif
 
-#include <QtCore/QSet>
-#include <QtCore/QVariant>
-#include <QtCore/QtPlugin>
+#include <vlc/vlc.h>
+
+#include "audiooutput.h"
+#include "audiodataoutput.h"
+#include "devicemanager.h"
+#include "effect.h"
+#include "effectmanager.h"
+#include "globaldescriptioncontainer.h"
+#include "libvlc.h"
+#include "mediaobject.h"
+#include "sinknode.h"
+//#include "videodataoutput.h"
+#include "videowidget.h"
+
+#ifndef PHONON_VLC_NO_EXPERIMENTAL
+#include "experimental/avcapture.h"
+#endif // PHONON_VLC_NO_EXPERIMENTAL
 
 Q_EXPORT_PLUGIN2(phonon_vlc, Phonon::VLC::Backend)
 
@@ -53,38 +63,48 @@ namespace Phonon
 namespace VLC
 {
 
-/**
- * Constructs the backend. Sets the backend properties, fetches the debug level from the
- * environment, initializes libVLC, constructs the device and effect managers, initializes
- * PulseAudio support.
- *
- * \param parent A parent object for the backend (passed to the QObject constructor)
- */
+Backend *Backend::self;
+
 Backend::Backend(QObject *parent, const QVariantList &)
     : QObject(parent)
-    , m_deviceManager(NULL)
-    , m_effectManager(NULL)
+    , m_deviceManager(0)
+    , m_effectManager(0)
 {
-    /* Backend information properties */
+    self = this;
+
+    // Backend information properties
     setProperty("identifier",     QLatin1String("phonon_vlc"));
     setProperty("backendName",    QLatin1String("VLC"));
     setProperty("backendComment", QLatin1String("VLC backend for Phonon"));
     setProperty("backendVersion", QLatin1String(PHONON_VLC_VERSION));
     setProperty("backendIcon",    QLatin1String("vlc"));
-    setProperty("backendWebsite", QLatin1String("http://gitorious.org/phonon/phonon-vlc"));
+    setProperty("backendWebsite", QLatin1String("https://projects.kde.org/projects/kdesupport/phonon/phonon-vlc"));
 
     // Check if we should enable debug output
     int debugLevel = qgetenv("PHONON_VLC_DEBUG").toInt();
     if (debugLevel > 3) { // 3 is maximum
         debugLevel = 3;
     }
-    m_debugLevel = (DebugLevel)debugLevel;
+    Debug::setMinimumDebugLevel((Debug::DebugLevel)((int) Debug::DEBUG_NONE - 1 - debugLevel));
 
-    /* Actual libVLC initialisation */
-    if (vlcInit(debugLevel)) {
-        logMessage(QString("Using VLC version %0").arg(libvlc_get_version()));
+    // Actual libVLC initialisation
+    if (LibVLC::init()) {
+        debug() << "Using VLC version %0" << libvlc_get_version();
     } else {
-        qWarning("Phonon::VLC::vlcInit: Failed to initialize VLC");
+#ifdef __GNUC__
+#warning TODO - this error message is about as useful as a cooling unit in the arctic
+#warning TODO - supposedly Phonon VLC should not make kabooom if libvlc fails to init, probably impossible though
+#endif
+        QMessageBox msg;
+        msg.setIcon(QMessageBox::Critical);
+        msg.setWindowTitle(tr("LibVLC failed to initialize"));
+        msg.setText(tr("Phonon's VLC backend failed to start."
+                       "\n\n"
+                       "This usually means a problem with your VLC installation,"
+                       " please report a bug with your distributor."));
+        msg.setDetailedText(LibVLC::errorMessage());
+        msg.exec();
+        fatal() << "Phonon::VLC::vlcInit: Failed to initialize VLC";
     }
 
     m_deviceManager = new DeviceManager(this);
@@ -94,69 +114,67 @@ Backend::Backend(QObject *parent, const QVariantList &)
     // Initialise PulseAudio support
     PulseSupport *pulse = PulseSupport::getInstance();
     pulse->enable();
-    connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)), SIGNAL(objectDescriptionChanged(ObjectDescriptionType)));
+    connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)),
+            SIGNAL(objectDescriptionChanged(ObjectDescriptionType)));
 #endif
 }
 
 Backend::~Backend()
 {
-//    vlcRelease();
+    if (LibVLC::self)
+        delete LibVLC::self;
+    if (GlobalAudioChannels::self)
+        delete GlobalAudioChannels::self;
+    if (GlobalSubtitles::self)
+        delete GlobalSubtitles::self;
 #ifdef PHONON_PULSESUPPORT
     PulseSupport::shutdown();
 #endif
 }
 
-/**
- * Creates a backend object of the desired class and with the desired parent. Extra arguments can be provided.
- *
- * \param c The class of object that is to be created
- * \param parent The object that will be the parent of the new object
- * \param args Optional arguments for the object creation
- * \return The desired object or NULL if the class is not implemented.
- */
-QObject *Backend::createObject(BackendInterface::Class c, QObject* parent, const QList<QVariant>& args)
+QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const QList<QVariant> &args)
 {
-    #ifndef PHONON_VLC_NO_EXPERIMENTAL
+#ifndef PHONON_VLC_NO_EXPERIMENTAL
     Phonon::Experimental::BackendInterface::Class cex = static_cast<Phonon::Experimental::BackendInterface::Class>(c);
 
     switch (cex) {
     case Phonon::Experimental::BackendInterface::AvCaptureClass:
-        logMessage("createObject() : AvCapture created - WARNING: Experimental!");
+        warning() << "createObject() : AvCapture created - WARNING: Experimental!";
         return new Experimental::AvCapture(parent);
     default:
         if ((quint32) cex >= (quint32) Phonon::Experimental::BackendInterface::VideoDataOutputClass)
-            logMessage("createObject() - experimental : Backend object not available");
+            warning() << "createObject() - experimental : Backend object not available";
     }
-    #endif // PHONON_VLC_NO_EXPERIMENTAL
+#endif // PHONON_VLC_NO_EXPERIMENTAL
 
     switch (c) {
     case MediaObjectClass:
-        return new VLCMediaObject(parent);
+        return new MediaObject(parent);
     case VolumeFaderEffectClass:
-        logMessage("createObject() : VolumeFaderEffect not implemented");
+        debug() << "createObject() : VolumeFaderEffect not implemented";
         break;
     case AudioOutputClass: {
-        AudioOutput *ao = new AudioOutput(this, parent);
+        AudioOutput *ao = new AudioOutput(parent);
         m_audioOutputs.append(ao);
         return ao;
     }
     case AudioDataOutputClass:
-        return new AudioDataOutput(this, parent);
-        logMessage("createObject() : AudioDataOutput created - WARNING: POSSIBLY INCORRECTLY IMPLEMENTED");
+        return new AudioDataOutput(parent);
+        warning() << "createObject() : AudioDataOutput created - WARNING: POSSIBLY INCORRECTLY IMPLEMENTED";
         break;
     case VisualizationClass:
-        logMessage("createObject() : Visualization not implemented");
+        error() << "createObject() : Visualization not implemented";
         break;
     case VideoDataOutputClass:
 //        return new Phonon::VLC::VideoDataOutput(this, parent);
-        logMessage("createObject() : VideoDataOutput not implemented");
+        error() << "createObject() : VideoDataOutput not implemented";
         break;
     case EffectClass:
         return new Effect(m_effectManager, args[0].toInt(), parent);
     case VideoWidgetClass:
         return new VideoWidget(qobject_cast<QWidget *>(parent));
     default:
-        logMessage("createObject() : Backend object not available");
+        error() << "createObject() : Backend object not available";
     }
     return 0;
 }
@@ -171,11 +189,6 @@ bool Backend::supportsOSD() const
     return true;
 }
 
-bool Backend::supportsFourcc(quint32 fourcc) const
-{
-    return true;
-}
-
 bool Backend::supportsSubtitles() const
 {
     return true;
@@ -253,13 +266,6 @@ QStringList Backend::availableMimeTypes() const
     return m_supportedMimeTypes;
 }
 
-/**
- * Returns a list of indexes for the desired object types. It specifies a list of objects
- * of a particular category that the backend knows about. These indexes can be used with
- * objectDescriptionProperties() to get the properties of a particular object.
- *
- * \param type The type of objects for the list
- */
 QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
 {
     QList<int> list;
@@ -267,6 +273,10 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
     int dev;
 
     switch (type) {
+    case Phonon::AudioChannelType: {
+        list << GlobalAudioChannels::instance()->globalIndexes();
+    }
+    break;
     case Phonon::AudioOutputDeviceType: {
         deviceList = deviceManager()->audioOutputDevices();
         for (dev = 0 ; dev < deviceList.size() ; ++dev)
@@ -279,14 +289,6 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
             list.append(deviceList[dev].id);
     }
     break;
-#ifndef PHONON_VLC_NO_EXPERIMENTAL
-    case Phonon::VideoCaptureDeviceType: {
-        deviceList = deviceManager()->videoCaptureDevices();
-        for (dev = 0 ; dev < deviceList.size() ; ++dev)
-            list.append(deviceList[dev].id);
-    }
-    break;
-#endif // PHONON_VLC_NO_EXPERIMENTAL
     case Phonon::EffectType: {
         QList<EffectInfo *> effectList = effectManager()->effects();
         for (int eff = 0; eff < effectList.size(); ++eff) {
@@ -294,32 +296,40 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
         }
     }
     break;
-    default:
-        break;
+    case Phonon::SubtitleType: {
+        list << GlobalSubtitles::instance()->globalIndexes();
+    }
+    break;
+    case Phonon::VideoCaptureDeviceType: {
+        deviceList = deviceManager()->videoCaptureDevices();
+        for (dev = 0 ; dev < deviceList.size() ; ++dev)
+            list.append(deviceList[dev].id);
+    }
+    break;
     }
 
     return list;
 }
 
-/**
- * Returns a list of properties for a particular object of the desired category.
- *
- * \param type The type of object for the index
- * \param index The index for the object of the desired type
- * \return The property list. If the object is inexistent, an empty list is returned.
- */
 QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescriptionType type, int index) const
 {
     QHash<QByteArray, QVariant> ret;
     QList<DeviceInfo> deviceList;
 
     switch (type) {
+    case Phonon::AudioChannelType: {
+        const AudioChannelDescription description = GlobalAudioChannels::instance()->fromIndex(index);
+        ret.insert("name", description.name());
+        ret.insert("description", description.description());
+    }
+    break;
     case Phonon::AudioOutputDeviceType: {
         deviceList = deviceManager()->audioOutputDevices();
         if (index >= 0 && index < deviceList.size()) {
             ret.insert("name", deviceList[index].name);
             ret.insert("description", deviceList[index].description);
             ret.insert("icon", QLatin1String("audio-card"));
+            ret.insert("isAdvanced", deviceList[index].isAdvanced);
         }
     }
     break;
@@ -329,28 +339,13 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
             ret.insert("name", deviceList[index].name);
             ret.insert("description", deviceList[index].description);
             ret.insert("icon", QLatin1String("audio-input-microphone"));
-#ifndef PHONON_VLC_NO_EXPERIMENTAL
+            ret.insert("isAdvanced", deviceList[index].isAdvanced);
             ret.insert("deviceAccessList", QVariant::fromValue<Phonon::DeviceAccessList>(deviceList[index].accessList));
-#endif // PHONON_VLC_NO_EXPERIMENTAL
             if (deviceList[index].capabilities & DeviceInfo::VideoCapture)
                 ret.insert("hasvideo", true);
         }
     }
     break;
-#ifndef PHONON_VLC_NO_EXPERIMENTAL
-    case Phonon::VideoCaptureDeviceType: {
-        deviceList = deviceManager()->videoCaptureDevices();
-        if (index >= 0 && index < deviceList.size()) {
-            ret.insert("name", deviceList[index].name);
-            ret.insert("description", deviceList[index].description);
-            ret.insert("icon", QLatin1String("camera-web"));
-            ret.insert("deviceAccessList", QVariant::fromValue<Phonon::DeviceAccessList>(deviceList[index].accessList));
-            if (deviceList[index].capabilities & DeviceInfo::AudioCapture)
-                ret.insert("hasaudio", true);
-        }
-    }
-#endif // PHONON_VLC_NO_EXPERIMENTAL
-    break;
     case Phonon::EffectType: {
         QList<EffectInfo *> effectList = effectManager()->effects();
         if (index >= 0 && index <= effectList.size()) {
@@ -363,23 +358,36 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
         }
     }
     break;
-    default:
-        break;
+    case Phonon::SubtitleType: {
+        const SubtitleDescription description = GlobalSubtitles::instance()->fromIndex(index);
+        ret.insert("name", description.name());
+        ret.insert("description", description.description());
+        ret.insert("type", description.property("type"));
+    }
+    break;
+    case Phonon::VideoCaptureDeviceType: {
+        deviceList = deviceManager()->videoCaptureDevices();
+        if (index >= 0 && index < deviceList.size()) {
+            ret.insert("name", deviceList[index].name);
+            ret.insert("description", deviceList[index].description);
+            ret.insert("icon", QLatin1String("camera-web"));
+            ret.insert("isAdvanced", deviceList[index].isAdvanced);
+            ret.insert("deviceAccessList", QVariant::fromValue<Phonon::DeviceAccessList>(deviceList[index].accessList));
+            if (deviceList[index].capabilities & DeviceInfo::AudioCapture)
+                ret.insert("hasaudio", true);
+        }
+    }
+    break;
     }
 
     return ret;
 }
 
-/**
- * Called when a connection between nodes is about to be changed
- *
- * \param objects A set of objects that will be involved in the change
- */
 bool Backend::startConnectionChange(QSet<QObject *> objects)
 {
     //FIXME
     foreach(QObject * object, objects) {
-        logMessage(QString("Object: %0").arg(object->metaObject()->className()));
+        debug() << "Object:" << object->metaObject()->className();
     }
 
     // There is nothing we can do but hope the connection changes will not take too long
@@ -388,56 +396,27 @@ bool Backend::startConnectionChange(QSet<QObject *> objects)
     return true;
 }
 
-/**
- * Connects two media nodes. The sink is informed that it should connect itself to the source.
- *
- * \param source The source media node for the connection
- * \param sink The sink media node for the connection
- * \return True if the connection was successful
- */
 bool Backend::connectNodes(QObject *source, QObject *sink)
 {
-    logMessage(QString("Backend connected %0 to %1")
-               .arg(source->metaObject()->className())
-               .arg(sink->metaObject()->className()));
+    debug() << "Backend connected" << source->metaObject()->className() << "to" << sink->metaObject()->className();
 
-    // Example:
-    // source = Phonon::VLC_MPlayer::MediaObject
-    // sink = Phonon::VLC_MPlayer::VideoWidget
-
-    // Example:
-    // source = Phonon::VLC_MPlayer::MediaObject
-    // sink = Phonon::VLC_MPlayer::AudioOutput
-
-    // Example:
-    // source = Phonon::VLC_MPlayer::MediaObject
-    // sink = Phonon::VLC_MPlayer::Effect
-
-    // Example:
-    // source = Phonon::VLC_MPlayer::Effect
-    // sink = Phonon::VLC_MPlayer::AudioOutput
-
-    // Example:
-    // source = Phonon::Experimental::AvCapture
-    // sink = Phonon::VLC::VideoWidget
-
-    SinkNode *sinkNode = qobject_cast<SinkNode *>(sink);
+    SinkNode *sinkNode = dynamic_cast<SinkNode *>(sink);
     if (sinkNode) {
-        PrivateMediaObject *mediaObject = qobject_cast<PrivateMediaObject *>(source);
+        MediaObject *mediaObject = qobject_cast<MediaObject *>(source);
         if (mediaObject) {
             // Connect the SinkNode to a MediaObject
             sinkNode->connectToMediaObject(mediaObject);
             return true;
         }
 
-        #ifndef PHONON_VLC_NO_EXPERIMENTAL
+#ifndef PHONON_VLC_NO_EXPERIMENTAL
         Experimental::AvCapture *avCapture = qobject_cast<Experimental::AvCapture *>(source);
         if (avCapture) {
             // Connect the SinkNode to a AvCapture
             sinkNode->connectToAvCapture(avCapture);
             return true;
         }
-        #endif // PHONON_VLC_NO_EXPERIMENTAL
+#endif // PHONON_VLC_NO_EXPERIMENTAL
 
         /*
         Effect *effect = qobject_cast<Effect *>(source);
@@ -448,40 +427,30 @@ bool Backend::connectNodes(QObject *source, QObject *sink)
         */
     }
 
-    logMessage(QString("Linking %0 to %1 failed")
-               .arg(source->metaObject()->className())
-               .arg(sink->metaObject()->className()),
-               Warning);
+    warning() << "Linking" << source->metaObject()->className() << "to" << sink->metaObject()->className() << "failed";
 
     return false;
 }
 
-/**
- * Disconnects two previously connected media nodes. It disconnects the sink node from the source node.
- *
- * \param source The source node for the disconnection
- * \param sink The sink node for the disconnection
- * \return True if the disconnection was successful
- */
 bool Backend::disconnectNodes(QObject *source, QObject *sink)
 {
-    SinkNode *sinkNode = qobject_cast<SinkNode *>(sink);
+    SinkNode *sinkNode = dynamic_cast<SinkNode *>(sink);
     if (sinkNode) {
-        PrivateMediaObject *const mediaObject = qobject_cast<PrivateMediaObject *>(source);
+        MediaObject *const mediaObject = qobject_cast<MediaObject *>(source);
         if (mediaObject) {
             // Disconnect the SinkNode from a MediaObject
             sinkNode->disconnectFromMediaObject(mediaObject);
             return true;
         }
 
-        #ifndef PHONON_VLC_NO_EXPERIMENTAL
+#ifndef PHONON_VLC_NO_EXPERIMENTAL
         Experimental::AvCapture *avCapture = qobject_cast<Experimental::AvCapture *>(source);
         if (avCapture) {
             // Disconnect the SinkNode from a AvCapture
             sinkNode->disconnectFromAvCapture(avCapture);
             return true;
         }
-        #endif // PHONON_VLC_NO_EXPERIMENTAL
+#endif // PHONON_VLC_NO_EXPERIMENTAL
 
         /*
         Effect *effect = qobject_cast<Effect *>(source);
@@ -495,78 +464,28 @@ bool Backend::disconnectNodes(QObject *source, QObject *sink)
     return false;
 }
 
-/**
- * Called after a connection between nodes has been changed
- *
- * \param objects Nodes involved in the disconnection
- */
 bool Backend::endConnectionChange(QSet<QObject *> objects)
 {
     foreach(QObject * object, objects) {
-        logMessage(QString("Object: %0").arg(object->metaObject()->className()));
+        debug() << "Object:" << object->metaObject()->className();
     }
 
     return true;
 }
 
-/**
- * \return The device manager that is associated with this backend object
- */
 DeviceManager *Backend::deviceManager() const
 {
     return m_deviceManager;
 }
 
-/**
- * \return The effect manager that is associated with this backend object.
- */
 EffectManager *Backend::effectManager() const
 {
     return m_effectManager;
 }
 
-/**
- * Return a debuglevel that is determined by the
- * PHONON_VLC_DEBUG environment variable.
- *
- * \li Warning - important warnings
- * \li Info    - general info
- * \li Debug   - gives extra info
- *
- * \return The debug level.
- */
-Backend::DebugLevel Backend::debugLevel() const
+Debug::DebugLevel Backend::debugLevel() const
 {
-    return m_debugLevel;
-}
-
-/**
- * Print a conditional debug message based on the current debug level
- * If obj is provided, classname and objectname will be printed as well
- *
- * \param message Debug message
- * \param priority Priority of the debug message, see debugLevel()
- * \param obj Who gives the message, or some object relevant for the message
- */
-void Backend::logMessage(const QString &message, int priority, QObject *obj) const
-{
-    if (debugLevel() > 0) {
-        QString output;
-        if (obj) {
-            // Strip away namespace from className
-            QString className(obj->metaObject()->className());
-            const int nameLength = className.length() - className.lastIndexOf(':') - 1;
-            className = className.right(nameLength);
-            output.sprintf("%s %s (%s %p)", message.toLatin1().constData(),
-                           obj->objectName().toLatin1().constData(),
-                           className.toLatin1().constData(), obj);
-        } else {
-            output = message;
-        }
-        if (priority <= (int)debugLevel()) {
-            qDebug() << QString("PVLC(%1): %2").arg(priority).arg(output);
-        }
-    }
+    return Debug::minimumDebugLevel();
 }
 
 }
